package sparqlinterpreter;

import java.util.ArrayList;
import sparqltree.SparqlSemanticTree;
import triskeloperationtree.JoinNode;
import triskeloperationtree.OperationTree;

public class Interpreter {

    private SparqlSemanticTree semanticTree;
    private PatternHandler patternHandler;
    private FunctionHandler functionHandler;
    private ArrayList<String> selectVariables;
    private OperationTree operationTree;
    private int index;

    public Interpreter() {
        this.semanticTree = null;
        this.patternHandler = null;
        this.functionHandler = null;
        this.operationTree = new OperationTree();
        this.selectVariables = new ArrayList<>();
        this.index = 0;
    }

    public void executeQuery(String sparqlQuery) {
        semanticTree = new SparqlSemanticTree(sparqlQuery);
        patternHandler = new PatternHandler(semanticTree);
        functionHandler = new FunctionHandler(semanticTree);
        analizequery();
    }

    public OperationTree getOperationTree(){
        return operationTree;
    }
    
    public ArrayList<String> getSelectVariables(){
        return selectVariables;
    }
    
    private void analizequery() {
        String token = semanticTree.getValue(index);
        index++;
        if (token.equalsIgnoreCase("SELECT")) {
            analizeSelectClause();
        }
    }

    private void analizeSelectClause() {
        String token = semanticTree.getValue(index);
        while (semanticTree.isVariable(index)) {
            selectVariables.add(semanticTree.getValue(index));
            index++;
        }
        analizeWhereClause();
    }

    private void analizeWhereClause() {
        if (semanticTree.getValue(index).equals("WHERE")) {
            index++;
        }
        index++;
        JoinNode joinNode = new JoinNode();
        operationTree.setRoot(joinNode);
        while (!semanticTree.getValue(index).equals("}")) {
            if (semanticTree.isTripletMember(index)) {
                index = patternHandler.getPatterns(index, joinNode);
            }
            if (semanticTree.isFunction(index)) {
                operationTree.setRoot(functionHandler.getFunctionNode(index));
                index = functionHandler.getFunctionEndIndex();
                index++;
            }
            if (semanticTree.getValue(index).equals(".")) {
                index++;
            }
        }

    }
}
