package sparqlinterpreter;

import patterns.Constant;
import patterns.Pattern;
import patterns.Operand;
import patternmatching.Query;
import patternmatching.QueryOperand;
import patternmatching.Term;
import patternmatching.Variable;
import patterns.What;
import sparqltree.SparqlSemanticTree;

public class PatternBuilder {

    SparqlSemanticTree semanticTree;

    public PatternBuilder(SparqlSemanticTree semanticTree) {
        this.semanticTree = semanticTree;
    }

    public Pattern getPattern(int subjectIndex, int predicateIndex, int objectIndex) {
        return queryToPattern(createQuery(subjectIndex, predicateIndex, objectIndex));
    }

    private Query createQuery(int subjectIndex, int predicateIndex, int objectIndex) {
        QueryOperand subject = getQueryOperand(subjectIndex);;
        QueryOperand predicate = getQueryOperand(predicateIndex);;
        QueryOperand object = getQueryOperand(objectIndex);;
        return new Query(subject, predicate, object);
    }

    private Pattern queryToPattern(Query query) {
        // TODO:
        // Integrar diccionario, y traducir Term a ID y Variable a What.
        Operand subject = getPatternOperand(query.getSubject());
        Operand predicate = getPatternOperand(query.getPredicate());
        Operand object = getPatternOperand(query.getObject());
        return new Pattern(subject, predicate, object);
    }

    private QueryOperand getQueryOperand(int index) {
        if (semanticTree.isVariable(index))
            return new Variable(semanticTree.getValue(index));
        else
            return new Term(semanticTree.getValue(index));
    }

    private Operand getPatternOperand(QueryOperand subject) {
        if (subject instanceof Variable)
            return new What(((Variable) subject).getName());
        else
            return new Constant(((Term) subject).getText().hashCode());
    }

}
