package sparqlinterpreter;

import java.util.ArrayList;
import sparqltree.SparqlSemanticTree;
import triskeloperationtree.FunctionNode;
import triskeloperationtree.PatternNode;

public class PatternHandler {

    PatternAnalizer patternAnalizer;
    PatternBuilder patternBuilder;

    public PatternHandler(SparqlSemanticTree semanticTree) {
        this.patternAnalizer = new PatternAnalizer(semanticTree);
        this.patternBuilder = new PatternBuilder(semanticTree);
    }
    
    public int getPatterns(int patternStartIndex, FunctionNode node){
        int patternEndIndex = patternAnalizer.analizePattern(patternStartIndex);
        int subjectIndex = patternAnalizer.getSubject();
        ArrayList<Integer> predicates = patternAnalizer.getPredicateList();
        ArrayList<Integer> objects = patternAnalizer.getObjectList();
        for (Integer predicateIndex : predicates) {
            for (Integer objectIndex : objects) {
                node.addNode(getPatternNodeFromPattern(subjectIndex, predicateIndex, objectIndex));
            }
        }
        return patternEndIndex;
    }
    
    public PatternNode getPatternNodeFromPattern(int subjectIndex, int predicateIndex, int objectIndex){
        return new PatternNode(patternBuilder.getPattern(subjectIndex, predicateIndex, objectIndex));
    }
    
}
