package sparqlinterpreter;

import static org.junit.Assert.assertTrue;
import org.junit.Test;
import patterns.Constant;
import patterns.Operand;
import patterns.Pattern;
import patterns.What;
import triskeloperationtree.FilterExistsNode;
import triskeloperationtree.FilterNotExistsNode;
import triskeloperationtree.FunctionNode;
import triskeloperationtree.JoinNode;
import triskeloperationtree.MinusNode;
import triskeloperationtree.Node;
import triskeloperationtree.OperationTree;
import triskeloperationtree.OptionalNode;
import triskeloperationtree.PatternNode;

public class FunctionCreationTest {

    @Test
    public void FilterExistsDetection() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . FILTER EXISTS { ?person foaf:name ?name }}");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof FilterExistsNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    @Test
    public void FilterNotExistsDetection() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . FILTER NOT EXISTS { ?person foaf:name ?name }}");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof FilterNotExistsNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    @Test
    public void MinusDetection() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . MINUS { ?person foaf:name ?name }}");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof MinusNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    @Test
    public void OptionalDetection() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . OPTIONAL { ?person foaf:name ?name }}");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof OptionalNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    @Test
    public void JoinDetection() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . ?person foaf:name ?name }");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof JoinNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    @Test
    public void MultipleFunctions() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . OPTIONAL { ?person foaf:name ?name } . OPTIONAL { ?person foaf:name ?mbox }}");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof OptionalNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    @Test
    public void MultipleFunctionsAndMultiplePatterns() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name {?person rdf:type  foaf:Person . ?person foaf:address  ?address OPTIONAL { ?person foaf:name ?name } . MINUS { ?person foaf:mbox ?mbox }}");
        OperationTree operationTree = interpreter.getOperationTree();
        assertTrue(operationTree.getRoot() instanceof MinusNode);
        printTree(operationTree.getRoot(), 0);
        System.out.println("----------------------");
    }

    private void printTree(Node node, int level) {
        String nodeType;
        if (node instanceof FunctionNode) {
            FunctionNode functionNode = (FunctionNode) node;
            nodeType = getNodeType(functionNode);
            for (int j = 0; j < level; j++) {
                System.out.print("  ");
            }
            System.out.print("|-> ");
            System.out.println(nodeType);
            for (int i = 0; i < functionNode.getChildNumber(); i++) {

                printTree(functionNode.getNode(i), level + 1);
            }
        } else {
            PatternNode patternNode = (PatternNode) node;
            for (int j = 0; j < level; j++) {
                System.out.print("  ");
            }
            System.out.print("|-> ");
            PatterntoString(patternNode.getPattern());
            System.out.println(" ");
        }
    }

    private String getNodeType(FunctionNode functionNode) {
        if (functionNode instanceof FilterExistsNode) {
            return "FilterExistsNode";
        } else if (functionNode instanceof FilterNotExistsNode) {
            return "FilterNotExistsNode";
        } else if (functionNode instanceof MinusNode) {
            return "MinusNode";
        } else if (functionNode instanceof OptionalNode) {
            return "OptionalNode";
        } else {
            return "JoinNode";
        }
    }

    private void PatterntoString(Pattern pattern) {
        showOperand(pattern.getSubject());
        showOperand(pattern.getPredicate());
        showOperand(pattern.getObject());
    }

    private void showOperand(Operand operand) {
        if (operand instanceof Constant) {
            System.out.print(Long.toString(((Constant) operand).getId()));
            System.out.print(" ");
        } else {
            System.out.print(((What) operand).getName());
            System.out.print(" ");
        }
    }
}
