package sparqlinterpreter;

import static org.junit.Assert.assertEquals;
import org.junit.Test;
import patterns.Constant;
import patterns.Pattern;
import patterns.Operand;
import patterns.What;
import triskeloperationtree.FunctionNode;
import triskeloperationtree.OperationTree;
import triskeloperationtree.PatternNode;

public class OperationTreeCreationTest extends InterpreterTest {

    @Test
    public void simplePatternTreeCreation() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?title WHERE {_:a :book ?title}");
        OperationTree operationTree = interpreter.getOperationTree();
        FunctionNode joinNode = (FunctionNode) operationTree.getRoot();
        assertEquals(1, joinNode.getChildNumber());
        PatternNode patternNode = (PatternNode) joinNode.getNode(0);
        PatterntoString(patternNode.getPattern());
    }

   @Test
    public void multiplePatternTreeCreation() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?title WHERE {_:a :book ?title. _:a :author ?author. _:a :ISBN ?ISBN}");
        OperationTree operationTree = interpreter.getOperationTree();
        FunctionNode joinNode = (FunctionNode) operationTree.getRoot();
        assertEquals(3, joinNode.getChildNumber());
        for (int i = 0; i < joinNode.getChildNumber(); i++) {
            PatternNode patternNode = (PatternNode) joinNode.getNode(i);
            PatterntoString(patternNode.getPattern());
        }
    }

    @Test
    public void multiplePredicatePatternTreeCreation() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name WHERE {_:a :knows ?know; :name ?name}");
        OperationTree operationTree = interpreter.getOperationTree();
        FunctionNode joinNode = (FunctionNode) operationTree.getRoot();
        assertEquals(2, joinNode.getChildNumber());
        for (int i = 0; i < joinNode.getChildNumber(); i++) {
            PatternNode patternNode = (PatternNode) joinNode.getNode(i);
            PatterntoString(patternNode.getPattern());
        }
    }

    @Test
    public void multipleObjectsPatternTreeCreation() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?x WHERE {?x :nick \"Alice\", \"Alice_\"}");
        OperationTree operationTree = interpreter.getOperationTree();
        FunctionNode joinNode = (FunctionNode) operationTree.getRoot();
        assertEquals(2, joinNode.getChildNumber());
        for (int i = 0; i < joinNode.getChildNumber(); i++) {
            PatternNode patternNode = (PatternNode) joinNode.getNode(i);
            PatterntoString(patternNode.getPattern());
        }
    }

    @Test
    public void multiplePredicateObjectPatternTreeCreation() {
        Interpreter interpreter = new Interpreter();
        interpreter.executeQuery("SELECT ?name WHERE {?x :nick \"Alice\", \"Alice_\"; :mbox ?mbox; :name ?name}");
        OperationTree operationTree = interpreter.getOperationTree();
        FunctionNode joinNode = (FunctionNode) operationTree.getRoot();
        assertEquals(4, joinNode.getChildNumber());
        for (int i = 0; i < joinNode.getChildNumber(); i++) {
            PatternNode patternNode = (PatternNode) joinNode.getNode(i);
            PatterntoString(patternNode.getPattern());
        }
    }

    private void PatterntoString(Pattern pattern) {
        showOperand(pattern.getSubject());
        showOperand(pattern.getPredicate());
        showOperand(pattern.getObject());
        System.out.println(" ");
    }

    private void showOperand(Operand operand) {
        if (operand instanceof Constant) {
            System.out.print(Long.toString(((Constant) operand).getId()));
            System.out.print(" ");
        } else {
            System.out.print(((What) operand).getName());
            System.out.print(" ");
        }
    }

}
